require 'fileutils'
require 'yaml'
require 'sinatra/base'
require 'mustache'
require 'redcarpet'
require 'json'
require 'nokogiri'
require 'digest'
require "rubygems"
require 'uri'

#TODO: Add support for tags
#TODO: Make stress tests
#TODO: Fix bug : day being off with time in file

class Solutus
    ARCHIVE = "archive"
    DEFAULT_TEMPLATE = "default"
    BLOG_TEMPLATE = "post"
    BUILD_PATH = "deploy"
    SITE_PATH = File.join(BUILD_PATH, "site")
    BLOG_BUILD_PATH = File.join(SITE_PATH, ARCHIVE)
    STATIC_PATH = "static"
    TEMPLATES_PATH = "templates"
    SITE_PAGES_PATH = File.join("pages", "site-pages")
    POSTS_PATH = File.join("pages", "posts")
    BLOG_DATE_FORMAT = "%b %e, %Y"
    BLOG_DATE_DATA = "%Y-%m-%d %I:%M:%S"
    SETTINGS_FILE = "settings.yml"
    CACHE = ".solutus-cache"
    PASSWORD_FILE = "password.txt"
    DAY_ENDINGS = ["th", "st", "nd", "rd", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "th", "st", "nd", "rd", "th", "th", "th", "th", "th", "th", "th", "st"]

    VERSION = '0.2.1'

    LOG_FILE = 'log'

    @@blog_urls = Hash.new
    @@page_urls = Array.new

    class SolutusStache < Mustache 
    end

    def self.load_settings
        begin
            yml_text = ""
            f = File.open(SETTINGS_FILE, "r")
            f.each_line do |line|
                yml_text += line
            end
            f.close
            YAML.load(yml_text)
        rescue
            puts "Not a Solutus Project (yet) --- settings.yml not found"
            false
        end
    end

    class Solutus_Server < Sinatra::Base
        CSS = File.read(File.join(File.dirname(File.expand_path(__FILE__)), '../resources', "style.css"))
        FILE_EXCLUDES = ['password.txt', '.gitignore', 'nohup.out', 'log']
        UPLOAD_DIR_SUFF = 'i'
        UPLOAD_DIR = File.join(STATIC_PATH, UPLOAD_DIR_SUFF)

        enable :sessions

        set :public_folder, SITE_PATH
        set :environment, :production
        
        configure do
            glob = Solutus.load_settings
            set :global_settings, glob
        end

        def path_to_resources
            File.join(File.dirname(File.expand_path(__FILE__)), '../resources')
        end

        class ServerStache < Mustache 
        end

        get "/" do
            send_file File.join(settings.public_folder, 'index.html')
        end

        post "/login" do
            if !params["password"]
                return 'no password'
            end
            if !File.file?(PASSWORD_FILE)
                render_login("Please set your password with <code>solutus password *****</code>")
            end
            given_password_hash = Digest::MD5.hexdigest params["password"]
            password = File.read(PASSWORD_FILE)
            puts given_password_hash.to_s
            puts "password on file:" + password.to_s
            if given_password_hash.to_s == password.to_s
                write_log("Logged in from #{request.ip}")
                session[:verified] = true
                redirect "/edit"
            else
                write_log("Wrong password from #{request.ip}")
                render_login("Wrong password")
            end
        end

        get "/reset" do
            if !session[:verified]
                return render_login("You were logged out.")
            end
            Solutus.command("build")
            redirect "/"
        end
    
        get "/edit" do
            if !session[:verified]
                return render_login("Please log in!")
            end
            render_index
        end

        get "/editpage" do
            if !session[:verified]
                return render_login("You were logged out.")
            end
            render_edit_page(params["page-path"])
        end

        post "/savepage" do
            if !session[:verified]
                return "error"
            end
            path = params["path"]

            f = File.open(path, "r")
            yml_text = ""
            f.each_line do |line|
                yml_text += line
            end
            f.close

            data = YAML.load(yml_text)
            data["content"] = params["content"]

            new_contents = YAML.dump(data)
            f = File.open(path, "w")
            f.write(new_contents)
            f.close

            Solutus.command("build")
            "saved page @ #{path}"
        end

        get "/newpage" do
            if !session[:verified]
                return "error"
            end
            dir = params['dir']
            filename = params['filename']
            dir = File.join(SITE_PAGES_PATH, dir)
            path = File.join(dir, filename) + '.yml'
            if !File.directory?(dir)
                FileUtils.mkdir_p(dir)
            end
            if File.file?(path)
                return 'error: file already exists'
            end
            f = File.new(path, 'w')
            title = 'Page Title'
            text = <<HERE
template: default
title: #{title}
content: |
    <h1>#{title}</h1>
    <p>
        Some Content
    </p>
HERE
            f.write(text)
            f.close
            write_log("Created page #{path} from #{request.ip}")
            redirect "/edit"
        end

        post "/newpost" do
            if !session[:verified]
                return render_login("You were logged out.")
            end
            title = params["post-title"].gsub(/[^a-zA-Z0-9\-]/,"").strip
            if title.empty?
                return "Error"
            end
            path = Solutus.command("new", "post", title)
            Solutus.command("build")
            render_edit_post(path)
        end

        get "/editpost" do
            if !session[:verified]
                return render_login("You were logged out.")
            end
            render_edit_post(*params["post-path"].split(","))
        end

        post "/savepost" do
            if !session[:verified]
                return "error"
            end
            path = params["path"]
            f = File.open(path, "r")
            yml_text = ""
            f.each_line do |line|
                yml_text += line
            end
            f.close
            data = YAML.load(yml_text)
            data["title"] = params["title"]
            data["subtitle"] = params["subtitle"]
            new_contents = YAML.dump(data)
            new_contents += "---\n"
            new_contents += params["markdown"]
            f = File.open(path, "w")
            f.write(new_contents)
            f.close
            Solutus.command("build")
            "saved post @ #{path}"
        end

        get "/editfile" do
            if !session[:verified]
                return render_login("You were logged out.")
            end
            path = params['path']
            if File.file?(path)
                render_edit_file(path)
            else
                'error, not a file'
            end
        end

        post "/savefile" do
            if !session[:verified]
                return 'error'
            end
            path = params['path']
            if FILE_EXCLUDES.include?(path.split("/"[-1]))
                write_log("security breach attempt @ #{path} from #{request.ip}")
                return 'security error'
            end
            content = params['content']
            f = File.open(path, 'w')
            f.write(content)
            f.close
            write_log("Edited #{path} from #{request.ip}")
            Solutus.command('build')
            'saved'
        end

        post "/upload" do
            if !session[:verified]
                return "error"
            end

            filename = URI.escape(params[:file][:filename])
            file = params[:file][:tempfile]
            dir = params[:dir] #Probably sanitizxe this

            unless File.directory?(UPLOAD_DIR)
                FileUtils.mkdir_p(UPLOAD_DIR)
            end

            unless File.directory?(File.join(UPLOAD_DIR, dir))
                FileUtils.mkdir_p(File.join(UPLOAD_DIR, dir))
            end

            path_to_file = File.join(UPLOAD_DIR, dir, filename)

            File.open(path_to_file, 'wb') do |f|
                f.write(file.read)
            end
            write_log("Uploaded #{path_to_file} from #{request.ip}")
            
            Solutus.command("build")

            link = settings.global_settings['domain'] + "/" + UPLOAD_DIR_SUFF + "/#{dir}/" +  filename
            render_local('base.html', {
                "content" => "<h1>Link</h1><strong><a href='http://#{link}'>#{link}</a></strong><p><a href='/edit'>back</a></p>"
            })
        end

        post "/publish" do
            if !session[:verified]
                return "error"
            end
            Solutus.command("build")
            "built"
        end

        get "/:page" do
            path = File.join(settings.public_folder, params['page'], 'index.html')
            if File.file?(path)
                send_file path
            else
                send_404
            end
        end

        get "/archive/:year/:month/:day/:title" do
            path = File.join(settings.public_folder, ARCHIVE,
                params['year'], params["month"], params["day"], params["title"],
            'index.html')
            if File.file?(path)
                send_file path
            else
                send_404
            end
        end

        def write_log(stuff)
            times = Time.now.getlocal('-07:00')
            if !File.file?(LOG_FILE)
                f = File.new(LOG_FILE, 'w')
                f.write("#{times} (PST) Creating log file\n")
                f.close
            end
            f = File.open(LOG_FILE, 'a')
            f.write("#{times} #{stuff}\n")
            f.close
        end
    
        def render_edit_file(path)
            #TODO: do smth diff for YML files...
            ServerStache.template = File.read(File.join(path_to_resources, "editFile.html"))
            result = ServerStache.new
            result[:content] = File.read(path)
            result[:path] = path
            result.render
        end

        def send_404
            send_file File.join(path_to_resources, "404.html")
        end

        def render_edit_page(path)
            f = File.open(File.join(path_to_resources, "editPage.html"), "r")
            template = ""
            f.each_line do |line|
                template += line
            end
            f.close

            f = File.open(path, "r")
            file_contents = ""
            f.each_line do |line|
                file_contents += line
            end
            data = YAML.load(file_contents)

            ServerStache.template = template
            result = ServerStache.new
            
            result[:css] = CSS
            result[:title] = data["title"]
            result[:path] = path
            link = path.split("/")[-1].split(".")[0]
            if ["error", "index"].include?(link)
                link = link + ".html"
            end
            result[:link] = link
            settings.global_settings.each do |key, val|
                result[key.to_sym] = val
            end
            page_html = Solutus.render_page(Solutus.templates, settings.global_settings, data)
            parser = Nokogiri::HTML(page_html)
            parser.at_css("#solutus-content")['contenteditable'] = true
            parser.at_css("#solutus-content")['style'] = "border: 1px black dashed" 
            result[:everything] = parser.at_css("#solutus-everything")
            result.render 
        end

        def render_edit_post(path, url="/")
            f = File.open(File.join(path_to_resources, "editBlog.html"), "r")
            template = ""
            f.each_line do |line|
                template += line
            end
            f.close

            SolutusStache.template = template
            result = SolutusStache.new
            
            f = File.open(path, "r")
            file_contents = ""
            f.each_line do |line|
                file_contents += line
            end
            data = YAML.load(file_contents)

            result[:css] = CSS
            result[:title] = data["title"]
            result[:date] = data["date"].strftime(BLOG_DATE_FORMAT)
            result[:markdown] = file_contents.split("---")[-1]
            result[:link] = url
            result[:canview] = url != "/"
            result[:path] = path
            result[:subtitle] = data["subtitle"]
            result.render
        end

        def render_login(message="")
            ServerStache.template = File.read(File.join(path_to_resources, "login.html"))
            result = ServerStache.new
            result[:message] = message
            result[:css] = CSS
            result.render
        end

        def render_index
            f = File.open(File.join(path_to_resources, "index.html"), "r")
            template = ""
            f.each_line do |line|
                template += line
            end
            f.close
            SolutusStache.template = template
            result = SolutusStache.new
            settings.global_settings.each do |key, val|
                result[key.to_sym] = val
            end
            dirs = Dir.entries(SITE_PAGES_PATH)
            puts dirs
            dir_res = ''
            dirs.each do |d|
                if File.directory?(File.join(SITE_PAGES_PATH, d)) && d != ".."
                    val = d + "/"
                    if val == '.'
                        val = "/"
                    end
                    dir_res += "<option value='#{d}'>#{val}</option>"
                end
            end
            result[:page_paths] = dir_res
            result[:blogs] = blog_select
            result[:pages] = page_select
            result[:css] = CSS
            result[:files] = render_files
            result[:file_upload_paths] = render_file_upload_dirs
            result[:version] = VERSION
            result.render
        end

        def render_local(f, data)
            f = File.open(File.join(path_to_resources, f), "r")
            template = ""
            f.each_line do |line|
                template += line
            end
            f.close
            SolutusStache.template = template
            result = SolutusStache.new
            settings.global_settings.each do |key, val|
                result[key.to_sym] = val
            end
            data.each do |key, val|
                result[key.to_sym] = val
            end
            result[:css] = CSS
            result.render
        end

        def page_select
            result = "<select name=\"page-path\">\n"
            Solutus.page_urls.each do |page|
                path = page["file_path"]
                title = page["title"]
                result += "<option value=\"#{path}\">#{title}</option>\n"
            end
            result += "</select>"
            result
        end

        def render_files 
            result = "<select name=\"path\">\n"
            dirs = [".", "static/css", "templates", "pages/site-pages"]
            dirs.each do |dir|
                result += "<optgroup label='#{dir}'>"
                if dir == "."
                    files = Dir.entries(dir).select {|f| !File.directory? f}
                else
                    files = Solutus.relative_path_to_all_files_in_dir(dir)
                end
                files.each do |filename|
                    if !FILE_EXCLUDES.include?(filename)
                        path = File.join(dir, filename)
                        if File.file?(path)
                            result += "<option value=\"#{path}\">#{filename}</option>\n"
                        end
                    end
                end
                result += "</optgroup>"
            end
            result += "</select>"
            result
        end

        def render_file_upload_dirs
            result = ''
            unless File.directory?(UPLOAD_DIR)
                FileUtils.mkdir_p(UPLOAD_DIR)
            end
            Dir.entries(UPLOAD_DIR).each do |d| 
                if  (d != '.' && d != '..') && File.directory?(File.join(UPLOAD_DIR, d))
                    result += "<option>#{d}</option>"
                end
            end
            result
        end

        def blog_select
            result = "<select name=\"post-path\">\n"
            Solutus.blog_urls.each do |yr, arr|
                sorted = arr.sort_by {|k| k["date"]} .reverse
                sorted.each do |blog|
                    title = blog["title"]
                    path = blog["file_path"]
                    url = blog["url"]
                    result += "<option value=\"#{path},#{url}\">#{title}</option>\n"
                end
            end
            result += "</select>"
            result
        end
    end

    def self.blog_urls
        @@blog_urls
    end

    def self.page_urls
        @@page_urls
    end

    def self.templates
        @@templates
    end

    def self.command(*args)
        start_time = Time.now
        if args.length >= 1
            if ["-v", "--version"].include?(args[0])
                puts VERSION
                return
            elsif args[0] == "deploy"
                deploy
            elsif args[0] == "build"
                build
            elsif args[0] == "serve"
                build #right now, the server won't werve unless its built. 
                      #consider serializing data in cache to save time.
                      #TODO: Stress test the build for speed.
                serve
                return
            elsif args[0] == "start"
                build 
                serve
                return
            elsif args[0] == "publish"
                build
                deploy
            else 
                if args.length >= 2
                    if args[0] == "password"
                        create_password(args[1])
                    elsif args[0] == "create"
                        new_project(args[1])
                    else
                        if args.length >= 3
                            if args[0] == "new"
                                if args[1] == "page"
                                    new_page(args[2])
                                elsif args[1] == "post"
                                    return new_post(args[2])
                                end
                            end
                        else
                            puts "Invalid arguments"
                            return
                        end
                    end
                else
                    puts "Unknown command."
                    return
                end
            end
        end

        end_time = Time.now
        diff = end_time - start_time
        puts "Completed in #{diff} seconds."
    end

    def self.deploy
        puts "Solutus is deploying"
        system load_settings["deploy-command"]
    end

    def self.serve
        puts "Solutus is serving"
        puts "View at http://localhost:4567/"
        Solutus_Server.run!
    end

    def self.build
        puts "Solutus is building the static site"
        settings_data = load_settings
        if settings_data == false
            return
        end
        
        if File.directory?(BUILD_PATH)
            FileUtils.remove_dir(BUILD_PATH)
        end
        Dir.mkdir(BUILD_PATH)
        Dir.mkdir(SITE_PATH)
        copy_static

        @@templates = Hash.new
        Dir.entries(TEMPLATES_PATH).each do |entry|
            next if entry == "." || entry == ".."
            puts "Loading template #{entry}"
            path = File.join(TEMPLATES_PATH, entry)
            if File.file?(path)
                contents = ""
                f = File.open(path, "r")
                f.each_line do |line|
                    contents += line
                end
                @@templates[entry.split(".")[0]] = contents
            end
        end

        yaml_data = Hash.new

        @@blog_urls = Hash.new
        Dir.entries(POSTS_PATH).each do |entry|
            next if entry == "." || entry == ".."
            path = File.join(POSTS_PATH, entry)
            f = File.open(path, "r")
            contents = ""
            f.each_line do |line|
                contents += line
            end
            f.close

            yaml_data[entry] = contents
            data = YAML.load(contents)
            date = data["date"]
            subtitle = data.fetch("subtitle", "")
            relative_dir = File.join(date.year.to_s, date.month.to_s, date.day.to_s, entry.split(".")[0])
            year = date.year.to_s
            if !@@blog_urls.key?(year)
                @@blog_urls[year] = Array.new
            end
            @@blog_urls[year].push({
                "title" => data["title"], 
                "url" => File.join(ARCHIVE, relative_dir),
                "file_path" => path,
                "date" => date,
                "subtitle" => subtitle
            })
        end

        recents_count = 3
        recents = ""
        get_recents(recents_count).reverse.each do |post|
            url = post["url"]
            title = post["title"]
            date = prettify_date(post["date"])
            subtitle = post["subtitle"]

            block = <<HERE
<div class="solutus-recents-block">
    <a class="solutus-blog-thumbnail" href="/#{url}">#{title}</a> - #{date}
    <div class="solutus-blog-subtitle"><i>#{subtitle}</i></div>
</div>
HERE
            recents += block
        end

        yaml_data.each do |entry, contents|
            path = File.join(POSTS_PATH, entry)
            data = YAML.load(contents)
            markdown_parsed = render_markdown(contents.split("---")[-1])
            data["content"] = markdown_parsed
            data["recents"] = recents
            date = data["date"]

            puts "Rendering blog post #{entry}"
            html = render_page(@@templates, settings_data, data, true)

            relative_dir = File.join(date.year.to_s, date.month.to_s, date.day.to_s, entry.split(".")[0])
            root_url = File.join(ARCHIVE, relative_dir, "index.html")

            year = date.year.to_s

            dir_path = File.join(BLOG_BUILD_PATH, relative_dir)
            if !File.directory?(dir_path)
                FileUtils.mkdir_p(dir_path)
            end
            file_path = File.join(SITE_PATH, root_url)
            puts "Created #{file_path}"
            f = File.new(file_path, "w")
            f.write(html)
            f.close()
        end    

        @@page_urls = Array.new
        relative_path_to_all_files_in_dir(SITE_PAGES_PATH).each do |entry|
            next if entry == "." || entry == ".."
            puts "Rendering page #{entry}"
            path = File.join(SITE_PAGES_PATH, entry)
            if File.file?(path)
                yml_text = ""
                f = File.open(path, "r")
                f.each_line do |line|
                    yml_text += line
                end
                data = YAML.load(yml_text)
                data["recents"] = recents
                html = render_page(@@templates, settings_data, data, false)
                new_dir = File.join(SITE_PATH, entry.split(".")[0])
                if ["error", "index"].include? entry.split(".")[0]
                    new_path = File.join(SITE_PATH, entry.split(".")[0] + ".html")
                else
                    if !File.directory?(new_dir)
                        FileUtils.mkdir_p(new_dir)
                    end
                    new_path = File.join(new_dir, "index.html")
                end
                f = File.new(new_path, "w")
                puts "Created #{new_path}"
                f.write(html)
                f.close
                @@page_urls.push({
                    "title" => data["title"],
                    "file_path" => path
                })
            end
        end    

        # Create blog archive page 
        content = ""
        @@blog_urls.each do |yr, arr|
            sorted = arr.sort_by {|k| k["date"]} .reverse
            content += "<h3>#{yr}</h3>\n<hr>\n"
            sorted.each do |hash|
                url = hash["url"]
                title = hash["title"]
                pretty_date = prettify_date(hash["date"])
                content += "<div class=\"solutus-blog-block\">\n"
                content += "    <div class=\"solutus-blog-title\">\n"
                content += "        <a href=\"/#{url}\">#{title}</a>\n"
                content += "    </div>\n"
                content += "    <div>#{pretty_date}</div>\n"
                content += "</div>\n"
            end
        end
        if File.directory?(File.join(SITE_PATH, ARCHIVE))
            path = File.join(SITE_PATH, ARCHIVE, "index.html")
            data = Hash.new
            data["title"] = "Archive"
            data["content"] = content
            data["recents"] = recents
            html = render_page(@@templates, settings_data, data, false)
            f = File.new(path, "w")
            f.write(html)
            f.close
            puts "Created blog archive @ #{path}"
        end

        # Mirror build
        if settings_data["mirror"]
            dir_path = settings_data["mirror"]
            FileUtils.rm_rf("#{dir_path}/.", secure: true)
            FileUtils.copy_entry(SITE_PATH, dir_path)
        end
    end

    def self.render_page(templates, default_data, data, blog=false)
        if data.key?("template")
            SolutusStache.template = templates[data["template"]]
        else
            SolutusStache.template = templates[DEFAULT_TEMPLATE]
        end
        result = SolutusStache.new
        default_data.each do |key, val|
            result[key.to_sym] = val || ""
        end
        data.each do |key, val|
            next if key == "template"
            if key != "title"
                val = wrap_element(key, val)
            end
            result[key.to_sym] = val || ""
        end
        result[:blog] = blog || false
        result[:year] = Time.now.year
        result.render
    end
    
    def self.new_page(title)
        title = title.gsub(/\s+/, "").gsub(/[^a-zA-Z0-9\-]/, "").strip
        if title.empty?
            puts "Bad name"
            return
        end
        if ["edit"].include?(title)
            puts "Can't use #{title} as a page name."
            return
        end
    
        path = File.join(SITE_PAGES_PATH, title + ".yml")
        if File.file?(path)
            puts "Page with that name already exists."
            return
        end

        text = <<HERE
template: default
title: #{title}
content: |
    <h1>#{title}</h1>
    <p>
        Some Content
    </p>
HERE
        f = File.new(path, "w")
        f.write(text)
        f.close
        puts "Created new site page #{title} at #{path}"
    end

    def self.new_post(title)
        i = 1
        new_title = title.gsub(/\s+/, "").gsub(/[^a-zA-Z0-9\-]/, "").strip
        if title.empty?
            puts "Empty title"
            return ""
        end
        while true
            path = File.join(POSTS_PATH, new_title + ".md")
            if File.file?(path)
                new_title = title + "#{i}"
                i += 1
            else
                break
            end
        end
        date = Time.now.strftime(BLOG_DATE_DATA)
        text = <<HERE
---
title: #{title}
date: #{date}
subtitle: Another new blog post
---
*This is a temporary blog post page!*
HERE
        f = File.new(path, "w")
        f.write(text)
        f.close
        puts "Created new blog post #{title} at #{path}"
        path
    end

    def self.copy_static
        Dir.entries(STATIC_PATH).each do |entry|
            next if entry == "." || entry == ".."
            path = File.join(STATIC_PATH, entry)
            if File.file?(path)
                puts "Copying static file #{path}"
                FileUtils.copy(path, File.join(SITE_PATH, entry))
            elsif File.directory?(path)
                puts "Recursively copying static dir #{path}"
                FileUtils.copy_entry(path, File.join(SITE_PATH, entry))
            end
        end
    end

    def self.prettify_date(date)
        date.strftime("%b %e") + DAY_ENDINGS[date.strftime("%e").to_i]
    end
        
    def self.get_recents(count)
        res = Array.new
        @@blog_urls.each do |yr, arr|
            res = res + arr.sort_by {|k| k["date"]}
        end
        res.last(count)
    end

    def self.correct_time(time)
        time.utc.localtime("+05:30")
    end

    def self.render_markdown(markdown)
        Redcarpet::Markdown.new(Redcarpet::Render::HTML.new).render(markdown)
    end

    def self.wrap_element(key, content)
        if content.is_a?(Time)
            content = correct_time(content).strftime(BLOG_DATE_FORMAT)
        end
        result = ""
        result += "<div id=\"solutus-#{key}\">\n"
        result += content
        result += "</div>"
        result
    end

    def self.relative_path_to_all_files_in_dir(dir)
        result = Array.new
        Dir.entries(dir).each do |entry|
            next if entry == "." || entry == ".."
            path = File.join(dir, entry)
            if File.file?(path)
                result.push(entry)
            elsif File.directory?(path)
                result = result + relative_path_to_all_files_in_dir(path).collect {|x| File.join(entry, x)}
            end
        end
        result
    end


    def self.create_password(password)
        content = Digest::MD5.hexdigest password
        if File.file?(PASSWORD_FILE)
            FileUtils.rm(PASSWORD_FILE)
        end
        f = File.new(PASSWORD_FILE, "w")
        f.write(content)
        f.close
        puts "Wrote password to password.txt"
    end

    def self.new_project(name)
        name = name.gsub(/[^a-zA-Z0-9\-]/, "").strip
        if name == ""
            puts "Invalid name"
            return
        end
        if File.directory?(name)
            puts "Invalid name, already a folder with that name"
            print "Overwrite? [Y/n]"
            if STDIN.gets.chomp == "Y"
                FileUtils.remove_dir(name)
            else 
                return
            end
        end
        
        Dir.mkdir(name)
        
        paths = Array.new
        ["deploy", "static", "templates", "pages", CACHE].each do |dir|
            paths.push(File.join(name, dir))
        end
        ["css", "js", "imgs"].each do |dir|
            paths.push(File.join(name, "static", dir))
        end
        ["posts", "site-pages"].each do |dir|
            paths.push(File.join(name, "pages", dir))
        end
        
        paths.each do |path|
            Dir.mkdir(path)
        end
        
        settings_file =  <<HERE
name: #{name}
author: Your Name
domain: www.yoursite.tld
deploy-command: aws s3 sync --acl public-read --sse --delete deploy/site/ s3://www.yoursite.tld
watermark: |
  <p style="text-align: center;">
      <small>Powered by <a href="https://github.com/jeromew21/solutus">Solutus</a></small>
  </p>
stylesheets: |
    <link rel="stylesheet" type="text/css" href="/css/styles.css" />
scripts: |
    <script></script>
recents-count: 3
advanced: false
HERE

        styles_file = <<HERE
body {
    background-color: #EEEEEE;
}
HERE

        default_file = <<HERE
<!DOCTYPE html>
<head>
    <title>{{title}}</title>
    {{{stylesheets}}}
</head>
<body>
    <div id="solutus-everything">
        {{#blog}}
            <h1>{{title}}</h1>
            {{{date}}}
        {{/blog}}
        {{{content}}}
        <h3>Recent Posts</h3>
        {{{recents}}}
        <small style="display: block; text-align: center;">&copy; {{year}} {{author}}</small>
        {{{watermark}}}
    </div>
    {{{scripts}}}
</body>
</html>
HERE

        datetime = Time.now.strftime(BLOG_DATE_DATA)
        test_file = <<HERE
title: Example Blog Post
date: #{datetime}
subtitle: an example post...
---
*This is an example blog post page!*
HERE

        index_file = <<HERE
template: default
title: Home page
type: html
content: |
    <h1>Hello World!</h1>
    <p>
        this is a test
    </p>
HERE

        files = {
            File.join(name, "settings.yml") => settings_file,
            File.join(name, ".gitignore") => PASSWORD_FILE + "\n",
            File.join(name, "static", "css", "styles.css") => styles_file,
            File.join(name, "templates", "default.html") => default_file,
            File.join(name, "pages", "posts", "test.md") => test_file,
            File.join(name, "pages", "site-pages", "index.yml") => index_file,
        }
        
        files.each do |filename, contents|
            puts "Created #{filename}"
            f = File.new(filename, "w")
            f.write(contents)
            f.close
        end

        puts "Created new project #{name}"
    end
end
